@@ -265,6 +265,10 @@ impl AggWhole {
             agg: AggMaker::make(spec)?,
         })
     }
+    /// clone, but make a whole new Agg
+    pub fn deep_clone(&self) -> Self {
+        Self::new(&self.spec).unwrap()
+    }
 }
 
 /// A list of AggCol
@@ -288,6 +292,14 @@ impl AggWholeList {
     pub fn len(&self) -> usize {
         self.v.len()
     }
+    /// clone, but make a all new Aggs
+    pub fn deep_clone(&self) -> Self {
+        let mut n = Self::new();
+        for x in &self.v {
+            n.push(&x.spec).unwrap();
+        }
+        n
+    }
     /// fmt
     pub fn fmt(&mut self, f: num::NumFormat) {
         for x in &mut self.v {
@@ -1261,6 +1273,7 @@ impl AggMaker {
     pub fn make_counter2(spec: &str, pattern: &str) -> Result<Box<dyn Counter>> {
         // we can't call init here, because mutex is already held by make2()
         //        Self::init()?;
+        // we need to split the list of counters into a separate init
         let mut name = "";
         let mut utf8 = false;
         if !spec.is_empty() {
