@@ -8,7 +8,7 @@
 //!
 //! ```
 //! use cdx::comp::LineCompList;
-//! use cdx::LookbackReader;
+//! use cdx::util::LookbackReader;
 //! let mut comp = LineCompList::new();
 //! comp.add("price,float")?;
 //! comp.add("quant,num")?;
@@ -20,14 +20,14 @@
 //! reader.getline()?;
 //! assert_eq!(comp.comp_cols(reader.curr_line(), reader.prev_line(1)), std::cmp::Ordering::Equal);
 //! assert_eq!(comp.comp_cols(reader.prev_line(2), reader.prev_line(1)), std::cmp::Ordering::Less);
-//! # Ok::<(), cdx::Error>(())
+//! # Ok::<(), cdx::util::Error>(())
 //! ```
 #![allow(clippy::float_cmp)]
 use crate::column::{get_col, NamedCol};
 use crate::expr::Expr;
-use crate::num::{fcmp, str_to_d_lossy, ulp_to_ulong, Junk, JunkType, JunkVal};
+use crate::num::{fcmp, ulp_to_ulong, Junk, JunkType, JunkVal};
 use crate::text::Text;
-use crate::{err, Error, Result, TextLine};
+use crate::util::{err, prerr_n, Error, LookbackReader, Result, TextLine};
 use lazy_static::lazy_static;
 use std::cmp::Ordering;
 use std::fmt;
@@ -1122,52 +1122,6 @@ impl CompMaker {
         }
         err!("No such compare type : '{}'", comp.ctype)
     }
-    /*
-        /// Create a Comparator from separate specs
-        pub fn make3(columns: &str, method: &str, pattern: &str) -> Result<Comparator> {
-            Self::init()?;
-            let mut ret = Comparator::default();
-            if !columns.is_empty() {
-                for x in columns.split('.') {
-                    ret.cols.push(NamedCol::new_from(x)?);
-                }
-            }
-            for x in method.split('.') {
-                // junk types
-                if x.eq_ignore_ascii_case("rev") {
-                    ret.reverse = true;
-                } else {
-                    ret.ctype = x.to_string();
-                }
-            }
-            ret.comp = Self::make_comp(&ret.ctype, pattern)?;
-            Ok(ret)
-        }
-        /// make a Compare from a spec
-        pub fn make_comp(ctype: &str, pattern: &str) -> Result<Box<dyn Compare>> {
-            Self::init()?;
-            let ctype = Self::resolve_alias(ctype);
-            let mm = COMP_MAKER.lock().unwrap();
-            for x in &*mm {
-                if ctype.eq_ignore_ascii_case(x.tag) {
-                    return (x.maker)(pattern);
-                }
-            }
-            err!("No such compare type : '{}'", ctype)
-        }
-        /// Create a Comparator from a full spec, i.e. "Columns,Method,Pattern""
-        pub fn make(spec: &str) -> Result<Comparator> {
-            if let Some((a, b)) = spec.split_once(',') {
-                if let Some((c, d)) = b.split_once(',') {
-                    Self::make3(a, c, d)
-                } else {
-                    Self::make3(a, b, "")
-                }
-            } else {
-                Self::make3(spec, "", "")
-            }
-        }
-    */
 }
 
 #[derive(Default, Debug)]
@@ -1179,6 +1133,7 @@ pub struct CompList {
 /// Ordered list of [LineComp]
 pub struct LineCompList {
     c: Vec<LineComp>,
+    value: Vec<u8>,
 }
 
 impl CompList {
@@ -1432,8 +1387,13 @@ impl LineCompList {
             self.c[0].fill_cache_line(item, value);
         }
     }
+    /// get the value previously set
+    pub fn get_value(&self) -> &[u8] {
+        &self.value
+    }
     /// set value fo later comparison
     pub fn set(&mut self, value: &[u8], delim: u8) -> Result<()> {
+        self.value = value.to_owned();
         if self.c.len() == 1 {
             self.c[0].set(value);
         } else {
@@ -1645,7 +1605,7 @@ impl Compare for CompareNumeric {
         num_cmp_signed(left, right) == Ordering::Equal
     }
     fn fill_cache(&self, item: &mut Item, value: &[u8]) {
-        item.cache = ulp_to_ulong(str_to_d_lossy(value));
+        item.cache = ulp_to_ulong(value.to_f64_lossy());
     }
     fn set(&mut self, value: &[u8]) {
         self.value = value.to_vec();
@@ -1680,23 +1640,23 @@ impl Compare for CompareEqual {
 
 impl Compare for Comparef64 {
     fn comp(&self, left: &[u8], right: &[u8]) -> Ordering {
-        fcmp(str_to_d_lossy(left), str_to_d_lossy(right))
+        fcmp(left.to_f64_lossy(), right.to_f64_lossy())
     }
     fn equal(&self, left: &[u8], right: &[u8]) -> bool {
-        str_to_d_lossy(left) == str_to_d_lossy(right)
+        left.to_f64_lossy() == right.to_f64_lossy()
     }
     fn fill_cache(&self, item: &mut Item, value: &[u8]) {
-        item.cache = ulp_to_ulong(str_to_d_lossy(value));
+        item.cache = ulp_to_ulong(value.to_f64_lossy());
         item.set_complete();
     }
     fn set(&mut self, value: &[u8]) {
-        self.value = str_to_d_lossy(value);
+        self.value = value.to_f64_lossy();
     }
     fn comp_self(&self, right: &[u8]) -> Ordering {
-        fcmp(self.value, str_to_d_lossy(right))
+        fcmp(self.value, right.to_f64_lossy())
     }
     fn equal_self(&self, right: &[u8]) -> bool {
-        self.value == str_to_d_lossy(right)
+        self.value == right.to_f64_lossy()
     }
 }
 
@@ -1893,7 +1853,7 @@ impl LineCompare for LineCompExpr {
         item.set_complete();
     }
     fn set(&mut self, value: &[u8]) {
-        self.value = str_to_d_lossy(value)
+        self.value = value.to_f64_lossy()
     }
     fn comp_self_cols(&mut self, right: &TextLine) -> Ordering {
         let value = self.exprs[0].eval(right);
@@ -1913,6 +1873,29 @@ impl LineCompare for LineCompExpr {
     }
 }
 
+/// return true if ordering is bad.
+/// print appropriate message to stderr
+pub fn comp_check(f: &LookbackReader, cmp: &mut LineCompList, unique: bool) -> bool {
+    let c = cmp.comp_cols(f.prev_line(1), f.curr_line());
+    let bad = match c {
+        Ordering::Less => false,
+        Ordering::Equal => unique,
+        Ordering::Greater => true,
+    };
+    if c == Ordering::Equal && unique {
+        eprintln!("Lines are equal when they should be unique.");
+    } else if bad {
+        eprintln!("Lines are out of order");
+    }
+    if bad {
+        eprint!("{} : ", f.line_number() - 1);
+        prerr_n(&[&f.prev_line(1).line]);
+        eprint!("{} : ", f.line_number());
+        prerr_n(&[&f.curr_line().line]);
+    }
+    bad
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
